unit UnitHtmlClip;

{
    purpose:
    extract the HTML code from and HTML Clip
    encode the HTML into an HTML Clip

    This code handles the 0.9 HTML Clip format from specifications
    NOTE: SourceURL is not included in this, but is commonly found in practice
}

interface

type THTMLCLip = class
    class function extractHTMLfromClip(clip : string ): string;
    class function HTMLtoHTMLClip(html : string) : ansistring;
    class function extractSourceURLFromClip(clip : string) : string;
end;

implementation

uses System.Classes, UnitToken, SysUtils, System.StrUtils, UnitMisc, System.AnsiStrings;

class function THTMLCLip.extractHTMLfromClip(clip : string ): string;
var strReader : TStringReader; // StreamReader was failing for ReadLine
    line : string;
    idxStart, idxEnd, sz : integer;

    function extractNumber(s : string) : Integer;
    var numstr : string;
    begin

         numstr := UnitToken.TokenString(s, ':');
         try
            result := strtoint(s);
         except
            result := -1;
         end;
    end;
begin
    strReader := TStringReader.Create(clip);

    line := strReader.ReadLine; // version
    line := strReader.ReadLine; // start HTML
    idxStart := extractNumber(line);
    line := strReader.ReadLine; // end HTML;
    idxEnd := extractNumber(line);
    sz :=  idxEnd - idxstart;

    result := System.StrUtils.MidStr(clip, idxStart+1, sz);

    myfree(strReader);
end;


class function THTMLCLip.HTMLtoHTMLClip(html : string) : ansistring;
const NL = #13#10;
    HTML_Header =
        'version:0.9' + NL +     // 13 bytes
        'StartHTML:00000000' + NL +         // 18 bytes
        'EndHTML:00000000' + NL +            // 16 bytes
        'StartFragment:00000000' + NL +     // 22 bytes
        'EndFragment:00000000' + NL;         // 20 bytes
var startHTML, endHTML : integer;
    startFrag, endFrag : integer;
    htmlutf8 : ansistring;
    term : ansistring;
    testhtml : string;
    function pad(i : integer) : ansistring;
    begin
        result := ansistring(Format('%.*d', [8, i]));
    end;
begin
    htmlutf8 := Utf8Encode(html);

    startHTML := length(HTML_Header);
    endHTML := length(HTML_Header) + length(htmlutf8);
    // use the BODY tag for the fragment
    // default the HTML tag otherwise
    term := '<body>';
    startFrag := pos(string(term), lowercase(html));
    startFrag := startFrag + length(HTML_Header);
    term := '</body>';
    endFrag := pos(string(term), lowercase(html)) + length(term);
    endFrag := endFrag + length(HTML_Header);

    if startFrag = -1 then begin
        term := '<html>';
        startFrag := pos(string(term), lowercase(html));
        term := '</html>';
        endFrag := pos(string(term), lowercase(html)) + length(term);
    end;


    result :=
        'version:0.9' + NL +     // 13 bytes
        'StartHTML:' +   pad(startHTML) + NL +         // 18 bytes
        'EndHTML:' + pad(endHTML)+ NL +            // 16 bytes
        'StartFragment:' + pad(StartFrag)   + NL +     // 22 bytes
        'EndFragment:'+  pad(endFrag)+ NL +         // 20 bytes
        htmlutf8;

    testhtml := THTMLCLip.extractHTMLfromClip(result);
    Assert(html = testhtml, 'HTML Clip endcode failed');
end;


class function THTMLCLip.extractSourceURLFromClip(clip : string) : string;
var
    strReader : TStringReader;
    line : string;
    cnt, headersize, strlen  : integer;

    function getLine(strReader : TStringReader; var cnt : integer) : string; inline;
    begin
        result := strReader.ReadLine; // version
        inc(cnt, length(result) + 2 {for CR LF});

        // NOTE: this will actually be 2 over when EOF is reached
    end;
const SOURCE_HEADER = 'sourceURL:';
begin
    cnt := 0;

    strReader := TStringReader.Create(clip);
    line := getLine(strReader, cnt); // version
    line := getLine(strReader, cnt); // start HTML
    headersize := 0;
    try
        Delete(line, 1, length('startHTML:'));
        headersize := strtoint(line);
    except
    end;
    line := getLine(strReader, cnt); // end HTML;

    // Expected next:
    // start fragment
    // end fragment
    // sourcURL {optional}

    result := '';

    if headersize = 0 then
        strlen := length(clip)
    else
        strlen := headersize;
    while (cnt < strlen) do begin
         line := getLine(strReader, cnt);
         if line = #0 then BREAK;

         if StartsText(SOURCE_HEADER, line) then begin
            Delete(line, 1, length(SOURCE_HEADER));
            result := line;
            break;
         end;
    end;

end;
end.
