unit UnitIgnoredClips;

// Each program can optionally ignore new clipboard clips by type

interface

uses Generics.Collections;

type TIgnoredMedia = ( IM_HTML = 1, IM_RICHTEXT = 2, IM_FILES = 4, IM_PICS = 8 );
type TIgnoredMediaSet = set of (IMS_HTML, IMS_RICHTEXT, IMS_FILES, IMS_PICS );
type TProgramList = TDictionary<string, integer>;
type TIgnoredClips = class(TObject)
    private
        class var programList : TProgramList;
        class procedure init;
        class procedure deinit;
        class procedure loadSettings;
        class procedure saveSettings;
    public
        class procedure GetAllIgnored(programName : string; var ignoredset : TIgnoredMediaSet);
        class function IsIgnored(programName : string; media : TIgnoredMedia ) : boolean;
        class procedure SetIgnored(programName : string; media : TIgnoredMedia; value : boolean);
        class procedure SaveNow;
end;

implementation

uses UnitMisc, INIFiles, System.SysUtils, System.Classes;


// public interface
class procedure TIgnoredClips.GetAllIgnored(programName : string; var ignoredset : TIgnoredMediaSet);
var i : integer;
begin
    self.programList.TryGetValue(programName,i);
    if (i and Integer(IM_HTML) <> 0) then ignoredset := ignoredset + [IMS_HTML];
    if (i and Integer(IM_RICHTEXT) <> 0) then ignoredset := ignoredset + [IMS_RICHTEXT];
    if (i and Integer(IM_FILES) <> 0) then ignoredset := ignoredset + [IMS_FILES];
    if (i and Integer(IM_PICS) <> 0) then ignoredset := ignoredset + [IMS_PICS];
end;

class function TIgnoredClips.IsIgnored(programName : string; media : TIgnoredMedia ) : boolean;
var i : integer;
begin
    result := false;
    if self.programList.TryGetValue(programName,i) then begin
        result := (integer(media) and i) > 0;
    end;
end;
class procedure TIgnoredClips.SetIgnored(programName : string; media : TIgnoredMedia; value : boolean);
var i : integer;
begin

    self.programList.TryGetValue(programName,i);
    if value then begin
        i := i or integer(media);
    end else begin
        i := i and (not integer(media) );
    end;
    self.programList.AddOrSetValue(programName, i);

end;
class procedure TIgnoredClips.SaveNow;
begin
    saveSettings;
end;

// Private
const PROGRAM_SECTION = 'Programs';
const PROGRAM_INI = 'programignore.ini';

class procedure TIgnoredClips.init;
begin
    programList := TProgramList.Create;
    self.loadSettings;
end;
class procedure TIgnoredClips.deinit;
begin
    self.saveSettings;
    freeAndNil(programList);
end;
class procedure TIgnoredClips.loadSettings;
var ini : TMemIniFile;
    i : integer;
    sl : TStringList;
    s : string;
begin
    ini := TMemIniFile.Create( UnitMisc.GetAppPath + PROGRAM_INI);

    sl := TStringList.create;
    ini.ReadSection(PROGRAM_SECTION, sl);

    for s in sl do begin
        i := ini.ReadInteger(PROGRAM_SECTION, s, 0);
        programList.AddOrSetValue(s, i);
    end;

    FreeAndNil(sl);
    FreeAndNil(ini);
end;
class procedure TIgnoredClips.saveSettings;
var s : string;
    i : integer;
    ini : TMemIniFile;
begin
    ini := TMemIniFile.Create( UnitMisc.GetAppPath + PROGRAM_INI);
    for s in programList.Keys do begin
        if programList.TryGetValue(s, i) then begin
            ini.WriteInteger(PROGRAM_SECTION, s, i);
        end;
    end;
    ini.UpdateFile;
    FreeAndNil(ini);
end;


initialization
begin
    TIgnoredClips.init;
end;

finalization
begin
   TIgnoredClips.deinit;
end;

end.
